/*= ====================================================================================
 *  Meteostanice - vysilaci modul	
 *==================================================================================================
*/

#define NRF9E5 1
#include "reg9e5.h"


#include "stdio.h"
//#define POWER      0                // 0=min power...3 = max power
#define HFREQ      0                // 0=433MHz, 1=868/915MHz
#define CHANNEL  115                // Channel number: f(MHz) = (422.4+CHANNEL/10)*(1+HFREQ)  433.9MHz (pro 115ku)
/*
433.2 - 108
433.7 - 113
434.2 - 118
434.7 - 123
*/

#define TIMEOUT    20               // 20ms time-out on LED
#define ON          1
#define OFF         0

#define VOLTSPY	2182					//1.95V pro 12 bit res a samo mereni

#define noACK 0
#define ACK 1
#define STATUS_REG_W 0x06 //000 0011 0
#define STATUS_REG_R 0x07 //000 0011 1
#define MEASURE_TEMP 0x03 //000 0001 1
#define MEASURE_HUMI 0x05 //000 0010 1
#define RESET 0x1e //000 1111 0

enum {TEMP,HUMI};


static volatile unsigned char t0lrel, t0hrel;

sbit LEDKA = P0 ^ 6;
sbit TST = P0 ^ 7;
sbit DDATA = P0 ^ 1;
sbit SCK = P0 ^ 0;


unsigned char tmp,i, Port, Citadlo, Service, Tick, Prijem, POW, error;
int Teplota, Vlhkost, Tlak;
unsigned int vysl;
unsigned int Napajeni;

//**********************************************************


//----------------------------------------------------------------------------------
char s_write_byte(unsigned char Xvalue)
//----------------------------------------------------------------------------------
// writes a byte on the Sensibus and checks the acknowledge
{
unsigned char i,error=0;

	P0_DIR &= 0xFD;			//nastavit jako out (do nuly)

	for (i=0x80;i>0;i/=2) 		//shift bit for masking
	{ if (i & Xvalue) DDATA=1; //masking value with i , write to SENSI-BUS
		else DDATA=0;
		SCK=1; 						//clk for SENSI-BUS
		_nop_();_nop_();_nop_(); //pulswith approx. 5 us
		_nop_();_nop_();_nop_();
		SCK=0;
		_nop_();_nop_();_nop_();
	}
   P0_DIR |= 0x02;			//nastavi jako vstup
	DDATA=1; //release DDATA-line
	SCK=1; //clk #9 for ack
	_nop_();_nop_();_nop_();
	error=DDATA; //check ack (DDATA will be pulled down by SHTXX)
	SCK=0;
	return error; //error=1 in case of no acknowledge
}

//----------------------------------------------------------------------------------
char s_read_byte(unsigned char ack, unsigned char Co)
//----------------------------------------------------------------------------------
// reads a byte form the Sensibus and gives an acknowledge in case of "ack=1"
{

unsigned char val=0;
unsigned int i=0;

	DDATA=1; //release DDATA-line

   P0_DIR |= 0x02;			//nastavi jako vstup
   
	if (Co == 0)
	{
		for (i=0x80;i>0;i/=2) //shift bit for masking
		{ SCK=1; //clk for SENSI-BUS
			_nop_();_nop_();_nop_(); //pulswith approx. 5 us
			_nop_();_nop_();_nop_();
			if (DDATA) val=(val | i); //read bit
			SCK=0;
			_nop_();_nop_();_nop_(); //pulswith approx. 5 us
			_nop_();_nop_();_nop_();
		}
	}

	if (Co == 1)				// CRC has to be read in reversed order
	{
		for (i=0x01;i<255;i*=2) //shift bit for masking
		{ SCK=1; //clk for SENSI-BUS
			_nop_();_nop_();_nop_(); //pulswith approx. 5 us
			_nop_();_nop_();_nop_();
			if (DDATA) val=(val | i); //read bit
			SCK=0;
			_nop_();_nop_();_nop_(); //pulswith approx. 5 us
			_nop_();_nop_();_nop_();
		}
	}

	P0_DIR &= 0xFD;			//nastavit jako out (do nuly)
	DDATA=!ack; //in case of "ack==1" pull down DDATA-Line
	SCK=1; //clk #9 for ack
	_nop_();_nop_();_nop_(); //pulswith approx. 5 us
	_nop_();_nop_();_nop_();
	SCK=0;
	P0_DIR |= 0x02;			//nastavi jako vstup
	DDATA=1; //release DDATA-line
	_nop_();_nop_();_nop_();
return val;
}

//----------------------------------------------------------------------------------
void s_transstart(void)
//----------------------------------------------------------------------------------
// generates a transmission start
//       _____         ________
// DDATA:     |_______|
//           ___     ___
// SCK : ___|   |___|   |______
{
	P0_DIR &= 0xFD;			//nastavit jako out (do nuly)
	DDATA=1; SCK=0; //Initial state
	_nop_();
	SCK=1;
	_nop_();
	DDATA=0;
	_nop_();
	SCK=0;
	_nop_();_nop_();_nop_();
	_nop_();_nop_();_nop_();
	SCK=1;
	_nop_();
	DDATA=1;
	_nop_();
	SCK=0;
}

//----------------------------------------------------------------------------------
void s_connectionreset(void)
//----------------------------------------------------------------------------------
// communication reset: DDATA-line=1 and at least 9 SCK cycles followed by transstart
//       _____________________________________________________         ________
// DDATA: 																	  |_______|
// 			_    _    _    _    _    _    _    _    _        ___     ___
// SCK : __| |__| |__| |__| |__| |__| |__| |__| |__| |______|   |___|   |______
{
unsigned char i;
	
	P0_DIR &= 0xFD;			//nastavit jako out (do nuly)

	DDATA=1; SCK=0; //Initial state
	for(i=0;i<9;i++) //9 SCK cycles
	{ SCK=1;
		_nop_();_nop_();_nop_();
		SCK=0;
		_nop_();_nop_();_nop_();
	}
	s_transstart(); //transmission start
}



//----------------------------------------------------------------------------------
char s_write_statusreg(unsigned char Co)
//----------------------------------------------------------------------------------
// writes the status register with checksum (8-bit)
{
//unsigned char error=0;
	s_transstart(); //transmission start
	error+=s_write_byte(STATUS_REG_W);//send command to sensor
	error+=s_write_byte(Co); //send value of status register
	return error; //error>=1 in case of no response form the sensor
}


//----------------------------------------------------------------------------------
int s_measure(unsigned char mode)
//----------------------------------------------------------------------------------
// makes a measurement (humidity/temperature) with checksum
{
//unsigned error=0;
unsigned int i;
int Vysledek;
unsigned char Cteny_bajt;

//	CRC = 0;				//inicializace
	s_transstart(); //transmission start
	switch(mode){ //send command to sensor
	case TEMP : error+=s_write_byte(MEASURE_TEMP);  break;
	case HUMI : error+=s_write_byte(MEASURE_HUMI);  break;
	default : break;
	}

	P0_DIR |= 0x02;			//nastavi jako vstup
	for (i=0;i<65535;i++) if(DDATA==0) break; //wait until sensor has finished the measurement
	if(DDATA) error+=1; // or timeout (~2 sec.) is reached
	Cteny_bajt = (s_read_byte(ACK,0) & 0xFF);
	Vysledek = Cteny_bajt;
//	Calc_CRC(Cteny_bajt);				//prepocita CRC
	Cteny_bajt = (s_read_byte(noACK,0) & 0xFF);	
//	Calc_CRC(Cteny_bajt);				//prepocita CRC
	Vysledek = ((Vysledek << 8) + Cteny_bajt); //read the second byte (LSB)
//	checksum =s_read_byte(noACK,1); //read checksum

return Vysledek;
}

//----------------------------------------------------------------------------------------
void calc_sth11(void)
//----------------------------------------------------------------------------------------
// calculates temperature [ ? C] and humidity [%RH]
// input : humi [Ticks] (12 bit)
// temp [Ticks] (14 bit)
// output: humi [%RH]
// temp [ ? C]
{ 
float rh_lin; // rh_lin: Humidity linear
float rh_true; // rh_true: Temperature compensated humidity
float t_C; // t_C : Temperature [ ? C]
float tlakf;

t_C = Teplota*0.01 - 40; //calc. temperature from ticks to [ ? C]
rh_lin=-0.0000028*Vlhkost*Vlhkost + 0.0405*Vlhkost - 4.0; //calc. humidity from ticks to [%RH]
rh_true=(t_C-25)*(0.01+0.00008*Vlhkost)+rh_lin; //calc. temperature compensated humidity [%RH]

Teplota = (int)(t_C*10);
Vlhkost = (int)(rh_true*10);
//Humidity = Humidity +10;

tlakf = 10* (0.033424*Tlak + 10.555);			//prepocet tlaku - asi jeste udelam zaokrouhleni
Tlak = (int)(tlakf);
Tlak = Tlak + 33;			//korekce pro spravny tlak

//**********************************************************
}

void Delay100us(volatile unsigned char n)				// zpozdeni
{
    unsigned char i;
    while(n--)
        for(i=0;i<35;i++)
            ;
}


unsigned char SpiReadWrite(unsigned char b)			// SPI komunikace
{
    EXIF &= ~0x20;                  // Clear SPI interrupt
    SPI_DATA = b;                   // Move byte to send to SPI data register
    while((EXIF & 0x20) == 0x00)    // Wait until SPI hs finished transmitting
        ;
    return SPI_DATA;
}


void ReceivePacket(unsigned int KolikCekat)						//prijme 4 pakety z TXu a je tam timeout
{
   
	TXEN = 0; 
   TRX_CE = 1;
   while((DR == 0) & (Tick < KolikCekat));
   if (Tick < KolikCekat)			//nebyl to timeout
   {
   	RACSN = 0;
	   SpiReadWrite(RRP);
		Port = SpiReadWrite(0);   			
		Napajeni = SpiReadWrite(0);   			
 		Citadlo = SpiReadWrite(0);
	 	Service = SpiReadWrite(0);
		RACSN = 1;
		Prijem = 1;		//neco prislo
	}
	else Prijem = 0;

   TRX_CE = 0;
}

void TransmitPacket(void)				// posle, co ma
{
	TXEN = 1;

    RACSN = 0;
  	SpiReadWrite(WTP);
  	SpiReadWrite(Teplota>>8);			
  	SpiReadWrite(Teplota);			
  	SpiReadWrite(Vlhkost>>8);		
	SpiReadWrite(Vlhkost);		
//  	SpiReadWrite(Napajeni>>8);		
  //	SpiReadWrite(Napajeni);		
  	SpiReadWrite(Tlak>>8);		
  	SpiReadWrite(Tlak);		
  	SpiReadWrite(Napajeni>>4);	
  	SpiReadWrite(Service);	
    
    RACSN = 1;
    TRX_CE = 1;
    Delay100us(1);
    TRX_CE = 0;
    while(DR == 0);				//ceka, az se to posle
}


void InitTimer(void)
{
    TR0 = 0;
    TMOD &= ~0x03;
    TMOD |= 0x01;                   // mode 1
    CKCON |= 0x08;                  // T0M = 1 (/4 timer clock)
    t0lrel = 0x17;                  // 1KHz tick...
    t0hrel = 0xFC;                  // ... = 65536-4e6/(4*1e3) = FC17h
    TF0 = 0;                        // Clear any pending Timer0 interrupts
    ET0 = 1;                        // Enable Timer0 interrupt
}

void Timer0ISR (void) interrupt 1
{
    TF0 = 0;                        // Clear Timer0 interrupt
    TH0 = t0hrel;                   // Reload Timer0 high byte
    TL0 = t0lrel;                   // Reload Timer0 low byte
    Tick++;
}


unsigned int Mer_1_kanal(unsigned char Kanal)       //meri jeden kanal ADC
{
unsigned char x, tlo, thi;
unsigned long vysledek = 0;

		for (x = 0; x <64; x++)                //delame 8 samplu
		{
        RACSN = 0;			        // Start ADC conversion na danem kanale
        SpiReadWrite(SAV  | Kanal);
        RACSN = 1;

        while(EOC == 0);			        // Wait until ADC conversion completes

        RACSN = 0;			        // Read Out ADC Value
        SpiReadWrite(RAD);
        tlo = SpiReadWrite(NOP);  //lo byte
        thi = SpiReadWrite(NOP);
        vysledek = vysledek + ((thi << 8) + tlo);
        RACSN = 1;
		}

	return(vysledek/64);
}



void Kalibruj_tick (void)						// pro 4kHz musi vychazet 1000 pro 8MHz xtal, 500 pro 4MHz
{
unsigned int per, h, per2;


	T2CON = 0x0D;			// nastaveni capture
		TR2 = 0;
		TH2 = 0;
		TL2 = 0;
		TR2 = 1;
		T2CON = (T2CON & 0xBF); //smazne priznak	

		while ((T2CON & 0x40) == 0);
		per = (RCAP2H << 8) + RCAP2L;
		T2CON = (T2CON & 0xBF); //smazne priznak	
		while ((T2CON & 0x40) == 0);
		per2 = (RCAP2H << 8) + RCAP2L;
		vysl = (per2 - per);
/*
		if (vysl > 600) TICK_DV = 0x02;			//kalibrace
		if ((vysl > 375) & (vysl < 450)) TICK_DV = 0x04;			//kalibrace			
		if (vysl <= 375) TICK_DV = 0x05;			//kalibrace			
*/

		if (vysl > 1200) TICK_DV = 0x02;			//kalibrace
		if ((vysl > 750) & (vysl < 900)) TICK_DV = 0x04;			//kalibrace			
		if (vysl <= 750) TICK_DV = 0x05;			//kalibrace			
		

/* 
- musi se udelat kalibrace v v TICK_DV
- Ttick = (1+hodnota)/fCLK
900-1100 je ok
1200 az 1400 dat hodnotu 2 (tzn +1 = 3)
800 - 900 - dat 4ku (tj 5)
pod 750 tak dat 5ku 

*/



}


void PowerSet(void)
{

//	POW = Mer_1_kanal(0);		//nemuzu mit externi trimr protoze to zere
	POW = 1;

    RACSN = 0;
    SpiReadWrite(WRC);              // Write RF config address 0
    SpiReadWrite(CHANNEL & 0xff);   // CHANNEL bit7..0
    // Change power defined by POWER above, to 433 or 868/915MHz defined by HFREQ and
    // bit8 of CHANNEL:
    SpiReadWrite(tmp | (POW<<2) | (HFREQ << 1) | ((CHANNEL >> 8) & 0x01));
    RACSN = 1;


}

void InitADC(unsigned char Akce)
{
    // Configure and turn on ADC
    RACSN = 0;
    SpiReadWrite(WAC);              // Write to ADC config
	 if (Akce == 1) SpiReadWrite(0x3D);             // Select AIN3, PWR_UP = 1, VFSEL=1 (externi);
	 else SpiReadWrite(0x31) ;						//vypni ADC
    SpiReadWrite(0x0B);             // Set RES_CTRL = 3, Right justified, 12 bitu
    RACSN = 1;
}

void Init(void)
{


    P0_DIR = 0x9E;                  // vse jako vstupy, 6ka LED, 1 out SCK, 0 = vystup  10111110(BE) nebo 9E(10011110)
//    P0 = 0x89;                      // P0.3 = 1 and P0.6=0 for the rec/tran selection
  
    SPICLK = 1;
    SPI_CTRL = 0x02;                // Connect internal SPI controller to Radio
    
    TH1 = 243;          // 9600@8MHz (when T1M=1 and SMOD=1)
    CKCON |= 0x30;      // T1M=1 (/4 timer clock) a TIM2 to same
    PCON = 0x80;        // SMOD=1 (double baud rate)
    SCON = 0x52;        // Serial mode1, enable receiver
    TMOD = 0x20;        // Timer1 8bit auto reload 
   TI = 0;
    TR1 = 1;            // Start timer1
//    P0_ALT |= 0x00;     // nic
    P0_ALT |= 0x04;     // TXD nastav - pro seriovku
	 P0_DRV = 0x60;			// horni pin pro led (bylo 40), pin5 pro tlak cidlo
    P0_DIR |= 0x02;     // P0.1 (RxD) is input


	CKLFCON = 0xCF; 				//5-3X OFF, 2, XO_DIRECT, 1-0 UPCLKFREQ   00 001 1 11

    RACSN = 0;
    SpiReadWrite(RRC | 0x09);
    tmp = SpiReadWrite(0) | 0x04;						//1110 0111 je default, ma byt 1100 1111
    
    RACSN = 1;

    RACSN = 0;
    SpiReadWrite(WRC | 0x09);			
    SpiReadWrite(0xCF);				// 8MHz
//    SpiReadWrite(0xC0);				
    RACSN = 1;

    // Configure Radio:
    RACSN = 0;
    SpiReadWrite(WRC | 0x03);       // Write to RF config address 3 (RX payload)
    SpiReadWrite(0x02);             // 2 byte RX payload
    SpiReadWrite(0x08);             // 8 byte TX payload
    RACSN = 1;

    RACSN = 0;
    SpiReadWrite(RRC | 0x01);       // Read RF config address 1
    tmp = SpiReadWrite(0) & 0xf0;   // Clear the power and frequency setting bits
    RACSN = 1;


    InitTimer();
    EA = 1;
}

void main(void)
{


   Init();
	InitADC(0);
	PowerSet();				//nastavi vykon
//	Kalibruj_tick();

//	Tlak = 0;

	while(1)
	{
	
	  	//P0 = 0xFF;
	  	 P0 = P0 | 0x20;
	//	P0|= 0x20;		//nastavi log 1 na tom napajeni tlak cidla 
		InitADC(1);						//zapne adc
		Delay100us(20);
		Napajeni = Mer_1_kanal (8);
		Tlak = Mer_1_kanal(0);
//		P0 = 00;
		P0 = P0 & 0xDF;
		//P0&= 0xDF;

	
	 	s_connectionreset();			// make softstart
		Teplota = s_measure(TEMP);
		Vlhkost = s_measure(HUMI);
		calc_sth11();
		
		SBUF = (Napajeni>>4);
		while (~TI);
		TI = 0;

		InitADC(0);						//vypne ADC

//	    TXEN = 1;
		for (i = 0; i < 2; i++)		//posle data 2x
		{
	    	TransmitPacket();
		   Delay100us(1);
		}


		while ((REGX_CTRL & 0x10) != 0);			//cekej, az se ten reg uvolni
		REGX_MSB = 0x00;							//cas idle cca 55 sec
//		REGX_LSB = 0x00;
		REGX_LSB = 0x02;
		REGX_CTRL = 0x09;						// zapis to do RTC registru, pis 3 bajt rtc
		while ((REGX_CTRL & 0x10) != 0);			//cekej, az se ten reg uvolni - podle datasheetu

		REGX_MSB = 0xBF;							//cas idle cca 180sec
		REGX_LSB = 0x20;
//		REGX_MSB = 0x27;							//cas idle cca 180sec
//		REGX_LSB = 0x10;


		REGX_CTRL = 0x0A;						// zapis to do RTC registru
		LEDKA = 1;
		while ((REGX_CTRL & 0x10) != 0);			//cekej, az se ten reg uvolni - podle datasheetu

		CK_CTRL = 0x02;						// mid power down?

// *********** tady se probudil
		LEDKA = 0;                               
		while ((REGX_CTRL & 0x10) != 0);			//cekej, az se ten reg uvolni
		REGX_CTRL = 0x0B;						// zakaz RTC WRTCDIS

	}
}

